@@ -16,19 +16,23 @@ import difflib  # For station name misspelling checks
 import datetime
 import requests
 import h5py
-import pandas as pd
 import numpy as np
 import matplotlib.pyplot as plt
 import matplotlib.cm
 import matplotlib.dates as mdates
 
-from scipy.ndimage.filters import uniform_filter1d
+# from scipy.ndimage.filters import uniform_filter1d
 try:
     from functools import lru_cache
 except ImportError:
     from backports.functools_lru_cache import lru_cache
+try:
+    import pandas as pd
+except ImportError:
+    print("Warning: pandas not installed. apertools.gps will fail")
+    print(" to use gps module, pip install pandas")
 
-import apertools
+import apertools.los
 import apertools.utils
 import apertools.sario
 import apertools.plotting
@@ -37,7 +41,10 @@ from apertools.log import get_log
 logger = get_log()
 
 # URL for ascii file of 24-hour final GPS solutions in east-north-vertical (NA12)
-GPS_BASE_URL = "http://geodesy.unr.edu/gps_timeseries/tenv3/NA12/{station}.NA12.tenv3"
+# GPS_BASE_URL = "http://geodesy.unr.edu/gps_timeseries/tenv3/NA12/{station}.NA12.tenv3"
+# UPDATE 4/20/20: noticed they changed it to
+GPS_BASE_URL = "http://geodesy.unr.edu/gps_timeseries/tenv3/plates/NA/{station}.NA.tenv3"
+GPS_FILE = GPS_BASE_URL.split("/")[-1]
 
 # Assuming the master station list is in the
 DIRNAME = os.path.dirname(os.path.abspath(__file__))
@@ -136,14 +143,21 @@ def load_station_enu(station,
                      end_date=None,
                      to_cm=True,
                      zero_mean=True,
-                     zero_start=False):
+                     zero_start=False,
+                     force_download=False):
     """Loads one gps station's ENU data since start_date until end_date
     as separate Series items
 
     Will scale to cm by default, and center the first data point
     to 0
     """
-    station_df = load_station_data(station, to_cm=to_cm, start_date=start_date, end_date=end_date)
+    station_df = load_station_data(
+        station,
+        to_cm=to_cm,
+        start_date=start_date,
+        end_date=end_date,
+        force_download=force_download,
+    )
 
     if zero_start:
         start_val = station_df[['east', 'north', 'up']].iloc[:10].mean()
@@ -160,6 +174,7 @@ def load_station_data(station_name,
                       start_date=START_DATE,
                       end_date=None,
                       download_if_missing=True,
+                      force_download=False,
                       to_cm=True):
     """Loads one gps station's ENU data since start_date until end_date as a dataframe
 
@@ -169,14 +184,22 @@ def load_station_data(station_name,
         start_date (datetime), default 2014-11-1, cutoff for beginning of GPS data
         end_date (datetime): default None, cut off for end of GPS data
         download_if_missing (bool): default True
+        force_download (bool): default False
     """
     station_name = station_name.upper()
-    gps_data_file = os.path.join(GPS_DIR, '%s.NA12.tenv3' % station_name)
+    gps_data_file = os.path.join(GPS_DIR, GPS_FILE.format(station=station_name))
+    if force_download:
+        try:
+            os.remove(gps_data_file)
+            logger.info(f"force removed {gps_data_file}")
+        except FileNotFoundError:
+            pass
     if not os.path.exists(gps_data_file):
-        logger.warning("%s does not exist.", gps_data_file)
         if download_if_missing:
-            logger.info("Downloading %s", station_name)
+            logger.info(f"Downloading {station_name} to {gps_data_file}")
             download_station_data(station_name)
+        else:
+            raise ValueError("{gps_data_file} does not exist, download_if_missing = False")
 
     df = pd.read_csv(gps_data_file, header=0, sep=r"\s+")
     clean_df = _clean_gps_df(df, start_date, end_date)
@@ -190,9 +213,9 @@ def _clean_gps_df(df, start_date=None, end_date=None):
     df['dt'] = pd.to_datetime(df['YYMMMDD'], format='%y%b%d')
 
     if start_date:
-        df_ranged = df[df['dt'] >= start_date]
+        df_ranged = df[df['dt'] >= pd.to_datetime(start_date)]
     if end_date:
-        df_ranged = df_ranged[df_ranged['dt'] <= end_date]
+        df_ranged = df_ranged[df_ranged['dt'] <= pd.to_datetime(end_date)]
     df_enu = df_ranged[['dt', '__east(m)', '_north(m)', '____up(m)']]
     df_enu = df_enu.rename(mapper=lambda s: s.replace('_', '').replace('(m)', ''), axis='columns')
     df_enu.reset_index(inplace=True, drop=True)
@@ -255,8 +278,8 @@ def plot_stations(image_ll=None,
 
     if mask_invalid:
         try:
-            stack_mask = apertools.sario.load_mask(
-                directory=directory, deformation_filename=full_path)
+            stack_mask = apertools.sario.load_mask(directory=directory,
+                                                   deformation_filename=full_path)
             image_ll[stack_mask] = np.nan
         except Exception:
             logger.warning("error in load_mask", exc_info=True)
@@ -343,9 +366,8 @@ def download_station_data(station_name):
     url = GPS_BASE_URL.format(station=station_name)
     response = requests.get(url)
 
-    stationfile = url.split("/")[-1]  # Just blah.NA12.tenv3
-    filename = "{}/{}".format(GPS_DIR, stationfile)
-    logger.info("Saving to {}".format(filename))
+    filename = "{}/{}".format(GPS_DIR, GPS_FILE.format(station=station_name))
+    logger.info(f"Saving {url} to {filename}")
 
     with open(filename, "w") as f:
         f.write(response.text)
@@ -355,14 +377,17 @@ def station_std(station, to_cm=True, start_date=START_DATE, end_date=None):
     """Calculates the sum of east, north, and vertical stds of gps"""
     dts, enu_df = load_station_enu(station, start_date=start_date, end_date=end_date, to_cm=to_cm)
     if enu_df.empty:
-        logger.warning("%s gps data returned an empty dataframe")
+        logger.warning(f"{station} gps data returned an empty dataframe")
         return np.nan
     return np.sum(enu_df.std())
 
 
-def plot_gps_enu(station=None, days_smooth=12, start_date=START_DATE, end_date=None):
+def plot_gps_enu(station=None,
+                 days_smooth=12,
+                 start_date=START_DATE,
+                 end_date=None,
+                 force_download=True):
     """Plot the east,north,up components of `station`"""
-
     def remove_xticks(ax):
         ax.tick_params(
             axis='x',  # changes apply to the x-axis
@@ -371,62 +396,79 @@ def plot_gps_enu(station=None, days_smooth=12, start_date=START_DATE, end_date=N
             top=False,  # ticks along the top edge are off
             labelbottom=False)
 
-    dts, enu_df = load_station_enu(station, start_date=start_date, end_date=end_date, to_cm=True)
+    dts, enu_df = load_station_enu(
+        station,
+        start_date=start_date,
+        end_date=end_date,
+        to_cm=True,
+        force_download=force_download,
+    )
     (east_cm, north_cm, up_cm) = enu_df[['east', 'north', 'up']].T.values
 
     fig, axes = plt.subplots(3, 1)
     axes[0].plot(dts, east_cm, 'b.')
     axes[0].set_ylabel('east (cm)')
     # axes[0].plot(dts, moving_average(east_cm, days_smooth), 'r-')
-    axes[0].plot(dts, pd.Series(east_cm).rolling(days_smooth, min_periods=10).mean(), 'r-')
+    axes[0].plot(dts, moving_average(east_cm), 'r-')
     axes[0].grid(True)
     remove_xticks(axes[0])
 
     axes[1].plot(dts, north_cm, 'b.')
     axes[1].set_ylabel('north (cm)')
-    axes[1].plot(dts, pd.Series(north_cm).rolling(days_smooth, min_periods=10).mean(), 'r-')
+    axes[1].plot(dts, moving_average(north_cm), 'r-')
     axes[1].grid(True)
     remove_xticks(axes[1])
 
     axes[2].plot(dts, up_cm, 'b.')
     axes[2].set_ylabel('up (cm)')
-    axes[2].plot(dts, pd.Series(up_cm).rolling(days_smooth, min_periods=10).mean(), 'r-')
+    axes[2].plot(dts, moving_average(up_cm), 'r-')
     axes[2].grid(True)
     # remove_xticks(axes[2])
 
     fig.suptitle(station)
+    plt.show(block=False)
 
     return fig, axes
 
 
 def load_gps_los_data(
-        geo_path=None,
-        los_map_file=None,
-        station_name=None,
-        to_cm=True,
-        zero_mean=True,
-        zero_start=False,
-        start_date=START_DATE,
-        end_date=None,
-        reference_station=None,
-        enu_coeffs=None,
+    los_map_file=None,
+    station_name=None,
+    to_cm=True,
+    zero_mean=True,
+    zero_start=False,
+    start_date=START_DATE,
+    end_date=None,
+    reference_station=None,
+    enu_coeffs=None,
+    force_download=False,
+    days_smooth=0,
 ):
     """Load the GPS timeseries of a station name
 
     Returns the timeseries, and the datetimes of the points
 
     This assumes that the los is AWAY from the satellite towards the ground.
-    We multiply the GPS data by -1 here so that ground moving upward
-    (towards the mostly-pointing-down satellite) shows positive value.
+
+    Ground moving upward (towards the mostly-pointing-down satellite) shows negate value,
+    as it is a smaller distance along the LOS (increase in LOS -> positive value)
     """
     if enu_coeffs is None:
         lon, lat = station_lonlat(station_name)
         enu_coeffs = apertools.los.find_enu_coeffs(
-            lon, lat, geo_path=geo_path, los_map_file=los_map_file)
+            lon,
+            lat,
+            los_map_file=los_map_file,
+        )
 
-    df = load_station_data(station_name, to_cm=to_cm, start_date=start_date, end_date=end_date)
-    # See docstring about negative 1 so vertical up ground gives a positive value
-    enu_data = -1 * df[['east', 'north', 'up']].T
+    df = load_station_data(
+        station_name,
+        to_cm=to_cm,
+        start_date=start_date,
+        end_date=end_date,
+        force_download=force_download,
+    )
+    enu_data = df[['east', 'north', 'up']].T
     los_gps_data = apertools.los.project_enu_to_los(enu_data, enu_coeffs=enu_coeffs)
 
     if zero_start:
@@ -436,9 +478,22 @@ def load_gps_los_data(
         logger.debug("Making GPS data 0 mean")
         los_gps_data = los_gps_data - np.mean(los_gps_data)
 
+    if days_smooth:
+        los_gps_data = moving_average(los_gps_data, days_smooth)
+
     if reference_station is not None:
-        dt_ref, losref = load_gps_los_data(geo_path, los_map_file, reference_station, to_cm,
-                                           zero_mean, zero_start, start_date, end_date)
+        dt_ref, losref = load_gps_los_data(
+            los_map_file,
+            reference_station,
+            to_cm,
+            zero_mean,
+            zero_start,
+            start_date,
+            end_date,
+            reference_station=None,
+            force_download=force_download,
+            days_smooth=days_smooth,
+        )
         return _merge_los(df['dt'], los_gps_data, dt_ref, losref)
 
     return df['dt'], los_gps_data
@@ -463,7 +518,8 @@ def moving_average(arr, window_size=7):
     """Takes a 1D array and returns the running average of same size"""
     if not window_size:
         return arr
-    return uniform_filter1d(arr, size=window_size, mode='nearest')
+    # return uniform_filter1d(arr, size=window_size, mode='nearest')
+    return np.array(pd.Series(arr).rolling(window_size).mean())
 
 
 def find_insar_ts(defo_filename='deformation.h5', station_name_list=[], window_size=1):
@@ -486,10 +542,13 @@ def find_insar_ts(defo_filename='deformation.h5', station_name_list=[], window_s
         dem_rsc = apertools.sario.load_dem_from_h5(defo_filename)
         row, col = apertools.latlon.nearest_pixel(dem_rsc, lon=lon, lat=lat)
         insar_ts_list.append(
-            get_stack_timeseries(
-                defo_filename, row, col, station=station_name, window_size=window_size))
+            get_stack_timeseries(defo_filename,
+                                 row,
+                                 col,
+                                 station=station_name,
+                                 window_size=window_size))
 
-    geolist = apertools.sario.load_geolist_from_h5(defo_filename)
+    geolist = apertools.sario.load_geolist_from_h5(defo_filename, dset=apertools.sario.STACK_DSET)
     return geolist, insar_ts_list
 
 
@@ -519,13 +578,15 @@ def _series_to_date(series):
     return pd.to_datetime(series).apply(lambda row: row.date())
 
 
-def create_insar_gps_df(geo_path,
+def create_insar_gps_df(geo_path="..",
+                        los_map_file="los_map.h5",
                         defo_filename='deformation.h5',
                         station_name_list=[],
                         reference_station=None,
                         window_size=1,
                         days_smooth_insar=None,
                         days_smooth_gps=None,
+                        create_diffs=True,
                         **kwargs):
     """Set days_smooth to None or 0 to avoid any data smoothing
 
@@ -541,10 +602,13 @@ def create_insar_gps_df(geo_path,
         )
 
     insar_df = create_insar_df(defo_filename, station_name_list, window_size, days_smooth_insar)
-    gps_df = create_gps_los_df(geo_path, station_name_list, days_smooth_gps)
+    gps_df = create_gps_los_df(los_map_file, station_name_list, days_smooth_gps)
     df = combine_insar_gps_dfs(insar_df, gps_df)
     if reference_station:
         df = subtract_reference(df, reference_station)
+    if create_diffs:
+        for s in station_name_list:
+            df[f"{s}_diff"] = df[f"{s}_gps"] - df[f"{s}_insar"]
     return _remove_bad_cols(df)
 
 
@@ -607,10 +671,10 @@ def create_insar_df(defo_filename='deformation.h5',
     return insar_df
 
 
-def create_gps_los_df(geo_path, station_name_list=[], days_smooth=30):
+def create_gps_los_df(los_map_file="los_map.h5", station_name_list=[], days_smooth=30):
     df_list = []
     for stat in station_name_list:
-        gps_dts, los_gps_data = load_gps_los_data(geo_path, stat)
+        gps_dts, los_gps_data = load_gps_los_data(los_map_file=los_map_file, station_name=stat)
 
         df = pd.DataFrame({"dts": _series_to_date(gps_dts)})
         df[stat + "_gps"] = moving_average(los_gps_data, days_smooth)
@@ -630,10 +694,8 @@ def create_gps_los_df(geo_path, station_name_list=[], days_smooth=30):
 
 def combine_insar_gps_dfs(insar_df, gps_df):
     # First constrain the date range to just the InSAR min/max dates
-    df = pd.DataFrame({
-        "dts":
-        pd.date_range(start=np.min(insar_df["dts"]), end=np.max(insar_df["dts"])).date
-    })
+    df = pd.DataFrame(
+        {"dts": pd.date_range(start=np.min(insar_df["dts"]), end=np.max(insar_df["dts"])).date})
     df = pd.merge(df, insar_df, on="dts", how="left")
     df = pd.merge(df, gps_df, on="dts", how="left", suffixes=("", "_gps"))
     # Now final df has datetime as index
@@ -656,8 +718,10 @@ def create_gps_enu_df(station_name_list=None,
 
     df_list = []
     for station in station_name_list:
-        dts, enu_df = load_station_enu(
-            station, start_date=start_date, end_date=end_date, to_cm=to_cm)
+        dts, enu_df = load_station_enu(station,
+                                       start_date=start_date,
+                                       end_date=end_date,
+                                       to_cm=to_cm)
 
         enu_df["dts"] = _series_to_date(dts)
         # Add station name to columns for post-merge identification
@@ -766,17 +830,19 @@ def _plot_slope_df(df, **kwargs):
     return fig, axes
 
 
-def _plot_line_df(df, ylim=None, share=True, days_smooth_gps=None, days_smooth_insar=None,
+def _plot_line_df(df,
+                  ylim=None,
+                  share=True,
+                  days_smooth_gps=None,
+                  days_smooth_insar=None,
                   **kwargs):
     """share is used to indicate that GPS and insar will be on same axes"""
-
     def _plot_smoothed(ax, df, column, days_smooth, marker):
-        ax.plot(
-            df[column].dropna().index,
-            df[column].dropna().rolling(days_smooth_gps).mean(),
-            marker,
-            linewidth=3,
-            label="%s day smoothed %s" % (days_smooth, column))
+        ax.plot(df[column].dropna().index,
+                df[column].dropna().rolling(days_smooth_gps).mean(),
+                marker,
+                linewidth=3,
+                label="%s day smoothed %s" % (days_smooth, column))
 
     columns = df.columns
     nrows = 1 if share else 2
@@ -846,11 +912,11 @@ def get_final_gps_insar_values(df, linear=True, as_df=False, velocity=True):
         return gps_cols, insar_cols, final_gps_vals, final_insar_vals
     else:
         final_val_station_order = [s.split('_')[0] for s in gps_cols]
-        return pd.DataFrame(
-            index=final_val_station_order, data={
-                'gps': final_gps_vals,
-                'insar': final_insar_vals
-            })
+        return pd.DataFrame(index=final_val_station_order,
+                            data={
+                                'gps': final_gps_vals,
+                                'insar': final_insar_vals
+                            })
 
 
 def _load_stations(igrams_dir=None, defo_filename=None, defo_full_path=None):
@@ -926,7 +992,6 @@ def plot_residuals_by_loc(df,
     df should be the timeseries df with "dts" as index created by create_insar_gps_df
     `which` argument is "diff","gps","insar", where 'diff' takes (gps - insar)
     """
-
     def _build_labels(df_merged, values):
         """Inclue station name and final value in label"""
         return ["{}:\n{:.2f}".format(name, val) for name, val in zip(df_merged.index, values)]
@@ -941,8 +1006,15 @@ def plot_residuals_by_loc(df,
     labels = _build_labels(df_merged, values)
     if title is None:
         title = "Final values of %s by location" % which
-    fig, ax = _plot_latlon_with_labels(
-        xs, ys, values, labels, title="", fig=fig, ax=ax, plot_scatter=plot_scatter, **plot_kwargs)
+    fig, ax = _plot_latlon_with_labels(xs,
+                                       ys,
+                                       values,
+                                       labels,
+                                       title="",
+                                       fig=fig,
+                                       ax=ax,
+                                       plot_scatter=plot_scatter,
+                                       **plot_kwargs)
     return df_merged, fig, ax
 
 
@@ -1011,13 +1083,19 @@ def plot_gps_east_by_loc(defo_filename,
     ys = df_merged['lat']
     vals = df_merged['east']
     # Note: reversed for ascending path so that red = west movement = toward satellite
-    cmap = apertools.plotting.make_shifted_cmap(
-        cmap_name=cmap_name, vmax=vals.max(), vmin=vals.min())
+    cmap = apertools.plotting.make_shifted_cmap(cmap_name=cmap_name,
+                                                vmax=vals.max(),
+                                                vmin=vals.min())
 
     first_date, last_date = east_df.index[0], east_df.index[-1]
     title = "east GPS movement from {} to {}".format(first_date, last_date)
-    fig, axes = _plot_latlon_with_labels(
-        xs, ys, vals, labels, title=title, cmap=cmap, **plot_kwargs)
+    fig, axes = _plot_latlon_with_labels(xs,
+                                         ys,
+                                         vals,
+                                         labels,
+                                         title=title,
+                                         cmap=cmap,
+                                         **plot_kwargs)
     return df_merged, fig, axes
 
 
