@@ -9,7 +9,6 @@ import pprint
 from datetime import datetime
 
 import apertools.latlon
-import apertools.utils
 from apertools.log import get_log
 logger = get_log()
 
@@ -130,31 +129,6 @@ class Sentinel(Base):
         self.map_overlay_kml = os.path.join(self._preview_folder, 'map-overlay.kml')
         self._lon_lat_overlay_coords = apertools.latlon.map_overlay_coords(self.map_overlay_kml)
 
-
-# TODO: in 30_20181107T092858_024480_02AF1F_BD80.iso.xml
-#                    <gmd:EX_BoundingPolygon>
-#                      <gmd:polygon>
-#                        <gml:Polygon srsName="http://www.opengis.net/gml/srs/epsg.xml#4326" srsDimension="2" gml:id="boundingPolygon">
-#                          <gml:exterior>
-#                            <gml:LinearRing>
-#                              <gml:pos>64.329208 -42.509876</gml:pos>
-#                              <gml:pos>64.784164 -47.771400</gml:pos>
-#                              <gml:pos>66.444023 -47.122295</gml:pos>
-#                              <gml:pos>65.972809 -41.522572</gml:pos>
-#                              <gml:pos>64.329208 -42.509876</gml:pos>
-#                            </gml:LinearRing>
-#                          </gml:exterior>
-#                        </gml:Polygon>
-#                      </gmd:polygon>
-#                    </gmd:EX_BoundingPolygon>
-# or
-#                      <gmd:eastBoundLongitude>
-#                        <gco:Decimal>-41.522572</gco:Decimal>
-#                      </gmd:eastBoundLongitude>
-#                      <gmd:southBoundLatitude>
-#                        <gco:Decimal>64.329208</gco:Decimal>
-#                      </gmd:southBoundLatitude>
-
     def _form_safe_dir(self, filename):
         """Get just the Sentinel product name without extensions, then add .SAFE"""
         # Strip '/' from end to start in case they pass "blahblah.SAFE/", or splitext[1] is ''
@@ -628,10 +602,10 @@ class Uavsar(Base):
         for p in self.POLARIZATIONS:
             shortname = shortname.replace(p, '')
 
-        ext = apertools.utils.get_file_ext(shortname)
+        ext = os.path.splitext(shortname)[1]
         # If this is a block we split up and names .1.int, remove that since
         # all have the same .ann file
-        shortname = re.sub('\.\d' + ext, ext, shortname)
+        shortname = re.sub(r'\.\d' + ext, ext, shortname)
         if ext == ".grd":
             # .int.grd is full ext
             full_ext = '.'.join(shortname.split('.')[1:])
@@ -704,7 +678,7 @@ def parse_ann_file(ann_filename, filename=None, ext=None, verbose=False):
     if not ext:
         if not filename:
             raise ValueError("Need either filename or ext")
-        ext = apertools.utils.get_file_ext(filename)
+        ext = os.path.splitext(filename)[1]
 
     if verbose:
         logger.info("Trying to load ann_data from %s", ann_filename)
