@@ -5,7 +5,6 @@ Email: scott.stanie@utexas.edu
 """
 
 from __future__ import division, print_function
-import collections
 import datetime
 import fileinput
 import glob
@@ -19,10 +18,10 @@ import warnings
 warnings.filterwarnings("ignore", category=FutureWarning)
 import h5py
 import matplotlib.pyplot as plt
-from PIL import Image
 
 import apertools.utils
 import apertools.parsers
+from .demloading import format_dem_rsc, load_dem_rsc, load_elevation
 from apertools.log import get_log
 logger = get_log()
 
@@ -36,8 +35,6 @@ _take_looks = apertools.utils.take_looks
 
 FLOAT_32_LE = np.dtype('<f4')
 COMPLEX_64_LE = np.dtype('<c8')
-INT_16_LE = np.dtype('<i2')
-INT_16_BE = np.dtype('>i2')
 
 SENTINEL_EXTS = ['.geo', '.cc', '.int', '.amp', '.unw', '.unwflat']
 UAVSAR_EXTS = [
@@ -111,7 +108,7 @@ UNW_FILENAME = "unw_stack.h5"
 CC_FILENAME = "cc_stack.h5"
 
 # dataset names for general 3D stacks
-STACK_DSET = "stack"
+STACK_DSET = "stack/1"
 STACK_MEAN_DSET = "mean_stack"
 STACK_FLAT_DSET = "deramped_stack"
 STACK_FLAT_SHIFTED_DSET = "deramped_shifted_stack"
@@ -127,32 +124,9 @@ DEM_RSC_DSET = "dem_rsc"
 GEOLIST_DSET = "geo_dates"
 INTLIST_DSET = "int_dates"
 
-RSC_KEY_TYPES = [
-    ('width', int),
-    ('file_length', int),
-    ('x_first', float),
-    ('y_first', float),
-    ('x_step', float),
-    ('y_step', float),
-    ('x_unit', str),
-    ('y_unit', str),
-    ('z_offset', int),
-    ('z_scale', int),
-    ('projection', str),
-]
-RSC_KEYS = [tup[0] for tup in RSC_KEY_TYPES]
-
-# in case only speciying rows/cols/steps, these always seem to be same
-DEFAULT_KEYS = {
-    'x_unit': 'degrees',
-    'y_unit': 'degrees',
-    'z_offset': 0,
-    'z_scale': 1,
-    'projection': 'LL',
-}
-
 
 def load_file(filename,
+              arr=None,
               downsample=None,
               looks=None,
               rsc_file=None,
@@ -166,6 +140,8 @@ def load_file(filename,
 
     Args:
         filename (str): path to the file to open
+        arr (ndarray): pre-allocated array of the correct size/dtype
+            TODO: (error if wrong size/type?)
         rsc_file (str): path to a dem.rsc file (if Sentinel)
         rsc_data (str): preloaded dem.rsc file as a dict
         ann_info (dict): data parsed from annotation file (UAVSAR)
@@ -184,6 +160,9 @@ def load_file(filename,
         ValueError: if sentinel files loaded without a .rsc file in same path
             to give the file width
     """
+    if not os.path.exists(filename):
+        raise FileNotFoundError(f"No such file or directory: '{filename}'")
+
     if downsample:
         if (downsample < 1 or not isinstance(downsample, int)):
             raise ValueError("downsample must be a positive integer")
@@ -220,6 +199,11 @@ def load_file(filename,
                 return f[list(f)[0]][:]
 
     if ext in IMAGE_EXTS:
+        try:
+            from PIL import Image
+        except ImportError:
+            print("Need PIL installed to save as image. `pip install pillow`")
+            raise
         return np.array(Image.open(filename).convert("L"))  # L for luminance == grayscale
 
     # Sentinel files should have .rsc file: check for dem.rsc, or elevation.rsc
@@ -254,9 +238,11 @@ def load_file(filename,
         raise ValueError("Need .rsc file or .ann file to load")
 
     if ext in BOOL_EXTS:
-        return _take_looks(load_bool(filename, rsc_data=rsc_data, rows=rows, cols=cols), *looks)
+        return _take_looks(load_bool(filename, arr=arr, rsc_data=rsc_data, rows=rows, cols=cols),
+                           *looks)
     elif ext in STACKED_FILES:
         stacked = load_stacked_img(filename,
+                                   arr=arr,
                                    rsc_data=rsc_data,
                                    ann_info=ann_info,
                                    rows=rows,
@@ -265,11 +251,16 @@ def load_file(filename,
         return stacked[..., ::downsample, ::downsample]
     elif is_complex(filename=filename, ext=ext):
         return _take_looks(
-            load_complex(filename, ann_info=ann_info, rsc_data=rsc_data, rows=rows, cols=cols),
-            *looks)
+            load_complex(filename,
+                         arr=arr,
+                         ann_info=ann_info,
+                         rsc_data=rsc_data,
+                         rows=rows,
+                         cols=cols), *looks)
     else:
         return _take_looks(
-            load_real(filename, ann_info=ann_info, rsc_data=rsc_data, rows=rows, cols=cols), *looks)
+            load_real(filename, arr=arr, ann_info=ann_info, rsc_data=rsc_data, rows=rows,
+                      cols=cols), *looks)
 
 
 # Make a shorter alias for load_file
@@ -364,16 +355,60 @@ def _assert_valid_size(data, cols):
     """
     error_str = "Invalid number of cols (%s) for file size %s." % (cols, len(data))
     # math.modf returns (fractional remainder, integer remainder)
-    assert math.modf(float(len(data)) / cols)[0] == 0, error_str
+    assert math.modf(float(data.size) / cols)[0] == 0, error_str
 
 
-def load_real(filename, rows=None, cols=None, ann_info=None, rsc_data=None, dtype=FLOAT_32_LE):
-    """Reads in real 4-byte per pixel files""
+def _load_binary1d(filename,
+                   arr=None,
+                   dtype=None,
+                   rows=None,
+                   cols=None,
+                   ann_info=None,
+                   rsc_data=None):
+    if arr is not None:
+        if dtype is not None:
+            assert (arr.dtype == np.dtype(dtype)), f"{arr.dtype} != {dtype}"
+        else:
+            dtype = arr.dtype
+        # https://github.com/numpy/numpy/blob/master/numpy/core/records.py#L896-L897
+        with open(filename, 'rb') as fd:
+            fd.readinto(arr.data)
+            data = arr
+    else:
+        data = np.fromfile(filename, dtype)
+
+    rows, cols = _get_file_rows_cols(rows=rows, cols=cols, ann_info=ann_info, rsc_data=rsc_data)
+    _assert_valid_size(data, cols)
+    return data, rows, cols
+
+
+def load_binary_img(filename,
+                    arr=None,
+                    rows=None,
+                    cols=None,
+                    ann_info=None,
+                    rsc_data=None,
+                    dtype=None):
+    data, rows, cols = _load_binary1d(
+        filename,
+        dtype=dtype,
+        arr=arr,
+        rows=rows,
+        cols=rows,
+        ann_info=ann_info,
+        rsc_data=rsc_data,
+    )
+    return data.reshape([-1, cols])
+
+
+def load_real(filename, arr=None, rows=None, cols=None, ann_info=None, rsc_data=None):
+    """Reads in real 4-byte per pixel files
 
     Valid filetypes: See sario.REAL_EXTS
 
     Args:
         filename (str): path to the file to open
+        arr (ndarray): pre-allocated array of the correct size/dtype
         rows (int): manually pass number of rows (overrides rsc/ann data)
         cols (int): manually pass number of cols (overrides rsc/ann data)
         rsc_data (dict): output from load_dem_rsc, gives width of file
@@ -383,40 +418,21 @@ def load_real(filename, rows=None, cols=None, ann_info=None, rsc_data=None, dtyp
         ndarray: float32 values for the real 2D matrix
 
     """
-    data = np.fromfile(filename, dtype)
-    rows, cols = _get_file_rows_cols(rows=rows, cols=cols, ann_info=ann_info, rsc_data=rsc_data)
-    _assert_valid_size(data, cols)
-    return data.reshape([-1, cols])
+    return load_binary_img(filename,
+                           arr=arr,
+                           rows=rows,
+                           cols=cols,
+                           ann_info=ann_info,
+                           rsc_data=rsc_data,
+                           dtype=FLOAT_32_LE)
 
 
-def load_complex(filename, rows=None, cols=None, ann_info=None, rsc_data=None, dtype=FLOAT_32_LE):
-    """Combines real and imaginary values from a filename to make complex image
-
-    Valid filetypes: See sario.COMPLEX_EXTS
-
-    Args:
-        filename (str): path to the file to open
-        rows (int): manually pass number of rows (overrides rsc/ann data)
-        cols (int): manually pass number of cols (overrides rsc/ann data)
-        rsc_data (dict): output from load_dem_rsc, gives width of file
-        ann_info (dict): data parsed from UAVSAR annotation file
-
-    Returns:
-        ndarray: imaginary numbers of the combined floats (dtype('complex64'))
-    """
-    data = np.fromfile(filename, dtype)
-    rows, cols = _get_file_rows_cols(rows=rows, cols=cols, ann_info=ann_info, rsc_data=rsc_data)
-    _assert_valid_size(data, cols)
-
-    real_data, imag_data = parse_complex_data(data, cols)
-    return combine_real_imag(real_data, imag_data)
-
-
-def load_bool(filename, rows=None, cols=None, ann_info=None, rsc_data=None, dtype=np.bool):
+def load_bool(filename, arr=None, rows=None, cols=None, ann_info=None, rsc_data=None):
     """Load binary boolean image
 
     Args:
         filename (str): path to the file to open
+        arr (ndarray): pre-allocated array of the correct size/dtype
         rows (int): manually pass number of rows (overrides rsc/ann data)
         cols (int): manually pass number of cols (overrides rsc/ann data)
         rsc_data (dict): output from load_dem_rsc, gives width of file
@@ -425,13 +441,41 @@ def load_bool(filename, rows=None, cols=None, ann_info=None, rsc_data=None, dtyp
     Returns:
         ndarray: imaginary numbers of the combined floats (dtype('complex64'))
     """
-    data = np.fromfile(filename, dtype)
-    rows, cols = _get_file_rows_cols(rows=rows, cols=cols, ann_info=ann_info, rsc_data=rsc_data)
-    _assert_valid_size(data, cols)
-    return data.reshape([-1, cols])
+    return load_binary_img(filename,
+                           arr=arr,
+                           rows=rows,
+                           cols=cols,
+                           ann_info=ann_info,
+                           rsc_data=rsc_data,
+                           dtype=np.bool)
+
+
+def load_complex(filename, arr=None, rows=None, cols=None, ann_info=None, rsc_data=None):
+    """Loads a Complex64 binary image
+
+    Args:
+        filename (str): path to the file to open
+        arr (ndarray): pre-allocated array of the correct size/dtype
+        rows (int): manually pass number of rows (overrides rsc/ann data)
+        cols (int): manually pass number of cols (overrides rsc/ann data)
+        rsc_data (dict): output from load_dem_rsc, gives width of file
+        ann_info (dict): data parsed from UAVSAR annotation file
+
+    Returns:
+        ndarray: complex64 values for the real 2D matrix
+
+    """
+    return load_binary_img(filename,
+                           arr=arr,
+                           rows=rows,
+                           cols=cols,
+                           ann_info=ann_info,
+                           rsc_data=rsc_data,
+                           dtype=COMPLEX_64_LE)
 
 
 def load_stacked_img(filename,
+                     arr=None,
                      rows=None,
                      cols=None,
                      rsc_data=None,
@@ -444,6 +488,10 @@ def load_stacked_img(filename,
     Format is two stacked matrices:
         [[first], [second]] where the first "cols" number of floats
         are the first matrix, next "cols" are second, etc.
+    Also called BIL, Band Interleaved by Line
+    See http://webhelp.esri.com/arcgisdesktop/9.3/index.cfm?topicname=BIL,_BIP,_and_BSQ_raster_files
+    for explantion
+
     For .unw height files, the first is amplitude, second is phase (unwrapped)
     For .cc correlation files, first is amp, second is correlation (0 to 1)
 
@@ -457,26 +505,16 @@ def load_stacked_img(filename,
     Returns:
         ndarray: dtype=float32, the second matrix (height, correlation, ...) parsed
         if return_amp == True, returns two ndarrays stacked along axis=0
-
-    Example illustrating how strips of data alternate:
-    reading unw (unwrapped phase) data
-
-    data = np.fromfile('20141128_20150503.unw', '<f4')
-
-    # The first section of data is amplitude data
-    # The amplitude has a different, larger range of values
-    amp = data[:cols]
-    print(np.max(amp), np.min(amp))
-    # Output: (27140.396, 118.341095)
-
-    # The next part of the data is a line of phases:
-    phase = data[cols:2*cols])
-    print(np.max(phase), np.min(phase))
-    # Output: (8.011558, -2.6779003)
     """
-    data = np.fromfile(filename, dtype)
-    rows, cols = _get_file_rows_cols(rows=rows, cols=cols, ann_info=ann_info, rsc_data=rsc_data)
-    _assert_valid_size(data, cols)
+    data, rows, cols = _load_binary1d(
+        filename,
+        dtype=dtype,
+        arr=arr,
+        rows=rows,
+        cols=rows,
+        ann_info=ann_info,
+        rsc_data=rsc_data,
+    )
 
     first = data.reshape((rows, 2 * cols))[:, :cols]
     second = data.reshape((rows, 2 * cols))[:, cols:]
@@ -506,19 +544,6 @@ def is_complex(filename=None, ext=None):
         return ext in COMPLEX_EXTS
 
 
-def parse_complex_data(complex_data, cols):
-    """Splits a 1-D array of real/imag bytes to 2 square arrays"""
-    # double check if I ever need rows
-    real_data = complex_data[::2].reshape([-1, cols])
-    imag_data = complex_data[1::2].reshape([-1, cols])
-    return real_data, imag_data
-
-
-def combine_real_imag(real_data, imag_data):
-    """Combines two float data arrays into one complex64 array"""
-    return real_data + 1j * imag_data
-
-
 def save(filename, data, normalize=True, cmap="gray", preview=False, vmax=None, vmin=None):
     """Save the numpy array in one of known formats
 
@@ -559,9 +584,6 @@ def save(filename, data, normalize=True, cmap="gray", preview=False, vmax=None,
             data = data / np.max(np.abs(data))
             vmin, vmax = -1, 1
         logger.info("previewing with (vmin, vmax) = (%s, %s)" % (vmin, vmax))
-        # from PIL import Image
-        # im = Image.fromarray(data)
-        # im.save(filename)
         if preview:
             plt.imshow(data, cmap=cmap, vmin=vmin, vmax=vmax)
             plt.colorbar()
@@ -708,21 +730,27 @@ def _load_deformation_npy(igram_path=None, filename=None, full_path=None, n=None
     return geolist, deformation
 
 
-def load_geolist_from_h5(h5file, dset=None):
+def load_geolist_from_h5(h5file, dset=None, parse=True):
     with h5py.File(h5file, "r") as f:
         if dset is None:
             geolist_str = f[GEOLIST_DSET][()].astype(str)
         else:
             geolist_str = f[dset].attrs[GEOLIST_DSET][()].astype(str)
 
-    return parse_geolist_strings(geolist_str)
+    if parse:
+        return parse_geolist_strings(geolist_str)
+    else:
+        return geolist_str
 
 
-def load_intlist_from_h5(h5file, dset=None):
+def load_intlist_from_h5(h5file, dset=None, parse=True):
     with h5py.File(h5file, "r") as f:
         date_pair_strs = f[INTLIST_DSET][:].astype(str)
 
-    return parse_intlist_strings(date_pair_strs)
+    if parse:
+        return parse_intlist_strings(date_pair_strs)
+    else:
+        return date_pair_strs
 
 
 def parse_geolist_strings(geolist_str):
@@ -779,7 +807,7 @@ def find_geos(directory=".", parse=True, filename=None):
 
 def _strip_geoname(name):
     """Leaves just date from format S1A_YYYYmmdd.geo"""
-    return name.replace('S1A_', '').replace('S1B_', '').replace('.geo', '')
+    return name.replace('S1A_', '').replace('S1B_', '').replace('.geo', '').replace('.vrt', '')
 
 
 def find_igrams(directory=".", parse=True, filename=None):
@@ -822,7 +850,13 @@ def save_dem_to_h5(h5file, dem_rsc, dset_name="dem_rsc", overwrite=True):
         f[dset_name] = json.dumps(dem_rsc)
 
 
-def save_geolist_to_h5(igram_path=None, out_file=None, overwrite=False, geo_date_list=None):
+def save_geolist_to_h5(
+    igram_path=None,
+    out_file=None,
+    dset_name=None,
+    geo_date_list=None,
+    overwrite=False,
+):
     if not check_dset(out_file, GEOLIST_DSET, overwrite):
         return
 
@@ -832,11 +866,20 @@ def save_geolist_to_h5(igram_path=None, out_file=None, overwrite=False, geo_date
     logger.debug("Saving geo dates to %s / %s" % (out_file, GEOLIST_DSET))
     with h5py.File(out_file, "a") as f:
         # JSON gets messed from doing from julia to h5py for now
-        # f[GEOLIST_DSET] = json.dumps(_geolist_to_str(geo_date_list))
-        f[GEOLIST_DSET] = _geolist_to_str(geo_date_list)
+        # f[GEOLIST_DSET] = json.dumps(geolist_to_str(geo_date_list))
+        if dset_name is not None:
+            f[dset_name].attrs[GEOLIST_DSET] = geolist_to_str(geo_date_list)
+        else:
+            f[GEOLIST_DSET] = geolist_to_str(geo_date_list)
 
 
-def save_intlist_to_h5(igram_path=None, out_file=None, overwrite=False, int_date_list=None):
+def save_intlist_to_h5(
+    igram_path=None,
+    dset_name=None,
+    out_file=None,
+    overwrite=False,
+    int_date_list=None,
+):
     if not check_dset(out_file, INTLIST_DSET, overwrite):
         return
 
@@ -845,14 +888,17 @@ def save_intlist_to_h5(igram_path=None, out_file=None, overwrite=False, int_date
 
     logger.info("Saving igram dates to %s / %s" % (out_file, INTLIST_DSET))
     with h5py.File(out_file, "a") as f:
-        f[INTLIST_DSET] = _intlist_to_str(int_date_list)
+        if dset_name is not None:
+            f[dset_name].attrs[INTLIST_DSET] = intlist_to_str(int_date_list)
+        else:
+            f[INTLIST_DSET] = intlist_to_str(int_date_list)
 
 
-def _geolist_to_str(geo_date_list):
+def geolist_to_str(geo_date_list):
     return np.array([d.strftime(DATE_FMT) for d in geo_date_list]).astype("S")
 
 
-def _intlist_to_str(int_date_list):
+def intlist_to_str(int_date_list):
     """Date pairs to Nx2 numpy array or strings"""
     return np.array([(a.strftime(DATE_FMT), b.strftime(DATE_FMT))
                      for a, b in int_date_list]).astype("S")
@@ -872,7 +918,7 @@ def load_geolist_intlist(directory, geolist_ignore_file=None, parse=True):
     Assumes that the .geo files are one diretory up from the igrams
     """
     int_date_list = find_igrams(directory, parse=parse)
-    geo_date_list = find_geos(apertools.utils.get_parent_dir(directory), parse=parse)
+    geo_date_list = find_geos(directory=apertools.utils.get_parent_dir(directory), parse=parse)
 
     if geolist_ignore_file is not None:
         ignore_filepath = os.path.join(directory, geolist_ignore_file)
@@ -885,8 +931,8 @@ def load_geolist_intlist(directory, geolist_ignore_file=None, parse=True):
 
 def ignore_geo_dates(geo_date_list, int_date_list, ignore_file="geolist_missing.txt", parse=True):
     """Read extra file to ignore certain dates of interferograms"""
-    ignore_geos = set(find_geos(ignore_file, parse=parse))
-    logger.info("Ignoreing the following .geo dates:")
+    ignore_geos = set(find_geos(filename=ignore_file, parse=parse))
+    logger.info("Ignoring the following .geo dates:")
     logger.info(sorted(ignore_geos))
     valid_geos = [g for g in geo_date_list if g not in ignore_geos]
     valid_igrams = [i for i in int_date_list if i[0] not in ignore_geos and i[1] not in ignore_geos]
@@ -1011,6 +1057,26 @@ def save_as_geotiff(outfile=None, array=None, rsc_data=None, nodata=0.0):
     out_raster = None
 
 
+def save_image_using_existing(arr, outname, input_fname, out_dtype=None, nodata=None):
+    """Writes out an array using the georeferencing data from `input_fname`"""
+    import rasterio as rio
+    with rio.open(input_fname) as src:
+        if (src.height, src.width) != arr.shape:
+            raise ValueError(f"{input_fname} must be same size as arr to use georeference data")
+
+        with rio.open(outname,
+                      "w",
+                      driver=src.driver,
+                      height=arr.shape[0],
+                      width=arr.shape[1],
+                      transform=src.transform,
+                      count=1,
+                      dtype=(out_dtype or arr.dtype),
+                      crs=src.crs,
+                      nodata=nodata) as dest:
+            dest.write(arr, 1)
+
+
 def save_as_vrt(filename=None,
                 array=None,
                 rows=None,
@@ -1023,7 +1089,6 @@ def save_as_vrt(filename=None,
                 band=None,
                 num_bands=None):
     """
-
     VRT options:
     SourceFilename: The name of the raw file containing the data for this band.
         The relativeToVRT attribute can be used to indicate if the
@@ -1044,14 +1109,24 @@ def save_as_vrt(filename=None,
     if outfile is None:
         raise ValueError("Need outfile or filename to save")
 
-    # Set geotransform (based on rsc data) and projection
+    # Get geotransform and project based on rsc data, or existing GDAL info
     if rsc_data is None:
         if rsc_file is None:
             # rsc_file = rsc_file if rsc_file else find_rsc_file(filename)
-            print("Warning: No .rsc file or data given")
-            geotrans = None
+            try:
+                ds = gdal.Open(filename)
+                geotrans = ds.GetGeoTransform()
+                srs = ds.GetSpatialRef()
+            except:
+                print(f"Warning: Cant get geotransform from {filename}, no .rsc file or data given")
+                geotrans = None
+                srs = gdal.osr.SpatialReference()
+                srs.SetWellKnownGeogCS("WGS84")
         else:
             rsc_data = load(rsc_file)
+            geotrans = rsc_to_geotransform(rsc_data)
+            srs = gdal.osr.SpatialReference()
+            srs.SetWellKnownGeogCS("WGS84")
 
     if array is not None:
         dtype = array.dtype
@@ -1067,7 +1142,16 @@ def save_as_vrt(filename=None,
 
     bytes_per_pix = np.dtype(dtype).itemsize
     total_bytes = os.path.getsize(filename)
-    assert rows == int(total_bytes / bytes_per_pix / cols)
+    if interleave is None or num_bands is None:
+        interleave, num_bands = get_interleave(filename, num_bands=num_bands)
+    if band is None:
+        # This will offset the start- only making the vrt point to phase
+        band = 1 if apertools.utils.get_file_ext(filename) in STACKED_FILES else 0
+
+    assert rows == int(
+        total_bytes / bytes_per_pix / cols /
+        num_bands), (f"rows = total_bytes / bytes_per_pix / cols / num_bands : "
+                     f"{rows} = {total_bytes} / {bytes_per_pix} / {cols} / {num_bands} ")
     # assert total_bytes == bytes_per_pix * rows * cols
 
     vrt_driver = gdal.GetDriverByName("VRT")
@@ -1075,21 +1159,13 @@ def save_as_vrt(filename=None,
     # out_raster = vrt_driver.Create(outfile, xsize=cols, ysize=rows, bands=1, eType=gdal_dtype)
     out_raster = vrt_driver.Create(outfile, xsize=cols, ysize=rows, bands=0)
 
-    if rsc_data is not None:
-        geotrans = rsc_to_geotransform(rsc_data)
+    if geotrans is not None:
         out_raster.SetGeoTransform(geotrans)
     else:
         print("Warning: No GeoTransform could be made/set")
 
-    srs = gdal.osr.SpatialReference()
-    srs.SetWellKnownGeogCS("WGS84")
     out_raster.SetProjection(srs.ExportToWkt())
 
-    if interleave is None or num_bands is None:
-        interleave, num_bands = get_interleave(filename, num_bands=num_bands)
-    if band is None:
-        band = 2 if apertools.utils.get_file_ext(filename) in STACKED_FILES else 1
-
     image_offset, pixel_offset, line_offset = get_offsets(
         dtype,
         interleave,
@@ -1173,7 +1249,11 @@ def create_derived_band(src_filename, outfile=None, src_dtype="CFloat32", desc=N
 
 
 def get_interleave(filename, num_bands=None):
-    """Returns band interleave format, and number of bands"""
+    """Returns band interleave format, and number of bands
+    
+    Band interleaved by line (BIL), band interleaved by pixel (BIP), and band sequential (BSQ)
+    https://desktop.arcgis.com/en/arcmap/10.3/manage-data/raster-and-images/bil-bip-and-bsq-raster-files.htm
+    """
     if num_bands == 1:
         # 1 band is always same: its just all pixels in a row
         return "BIP", 1
@@ -1191,12 +1271,12 @@ def get_interleave(filename, num_bands=None):
 
 def get_offsets(dtype, interleave, band, width, length, num_bands):
     """
-    From ISCE Image.py:
+    From ISCE Image.py
     """
     bytes_per_pix = np.dtype(dtype).itemsize
     if interleave == "BIL":
         return (
-            band * width * bytes_per_pix,  # ImageOFfset
+            band * width * bytes_per_pix,  # ImageOffset
             bytes_per_pix,  # PixelOffset
             num_bands * width * bytes_per_pix,  # LineOffset
         )
@@ -1294,25 +1374,6 @@ def make_cmy_colortable():
     # return out
 
 
-#     def memMap(self, mode='r', band=None):
-#         if self.scheme.lower() == 'bil':
-#             immap = np.memmap(self.filename, self.toNumpyDataType(), mode,
-#                             shape=(self.coord2.coordSize , self.bands, self.coord1.coordSize))
-#             if band is not None:
-#                 immap = immap[:, band, :]
-#         elif self.scheme.lower() == 'bip':
-#             immap = np.memmap(self.filename, self.toNumpyDataType(), mode,
-#                               shape=(self.coord2.coordSize, self.coord1.coordSize, self.bands))
-#             if band is not None:
-#                 immap = immap[:, :, band]
-#         elif self.scheme.lower() == 'bsq':
-#             immap = np.memmap(self.filename, self.toNumpyDataType(), mode,
-#                         shape=(self.bands, self.coord2.coordSize, self.coord1.coordSize))
-#             if band is not None:
-#                 immap = immap[band, :, :]
-#         return immap
-
-
 def numpy_to_gdal_type(np_dtype):
     from osgeo import gdal_array, gdalconst
     if np.issubdtype(bool, np_dtype):
@@ -1328,8 +1389,8 @@ def gdal_to_numpy_type(gdal_dtype=None, band=None):
     return gdal_array.GDALTypeCodeToNumericTypeCode(gdal_dtype)
 
 
-# TODO: this dont work to add a colorbar to grayscale tif...
 def testt(fn):
+    # TODO: not quite working to add a colorbar to grayscale tif...
     import gdal
     ds = gdal.Open(fn, 1)
     band = ds.GetRasterBand(1)
@@ -1361,131 +1422,6 @@ def make_unw_vrt(unw_filelist=None, directory=None, output="unw_stack.vrt", ext=
             print(line.replace("<SourceBand>1</SourceBand>", "<SourceBand>2</SourceBand>"), end='')
 
 
-def load_dem_rsc(filename, lower=False, **kwargs):
-    """Loads and parses the .dem.rsc file
-
-    Args:
-        filename (str) path to either the .dem or .dem.rsc file.
-            Function will add .rsc to path if passed .dem file
-        lower (bool): make keys of the dict lowercase
-
-    Returns:
-        dict: dem.rsc file parsed out, keys are all caps
-
-    example file:
-    WIDTH         10801
-    FILE_LENGTH   7201
-    X_FIRST       -157.0
-    Y_FIRST       21.0
-    X_STEP        0.000277777777
-    Y_STEP        -0.000277777777
-    X_UNIT        degrees
-    Y_UNIT        degrees
-    Z_OFFSET      0
-    Z_SCALE       1
-    PROJECTION    LL
-    """
-
-    # Use OrderedDict so that upsample_dem_rsc creates with same ordering as old
-    output_data = collections.OrderedDict()
-    # Second part in tuple is used to cast string to correct type
-
-    rsc_filename = '{}.rsc'.format(filename) if not filename.endswith('.rsc') else filename
-    with open(rsc_filename, 'r') as f:
-        for line in f.readlines():
-            for field, num_type in RSC_KEY_TYPES:
-                if line.startswith(field.upper()):
-                    output_data[field] = num_type(line.split()[1])
-
-    if lower:
-        output_data = {k.lower(): d for k, d in output_data.items()}
-    return output_data
-
-
-def format_dem_rsc(rsc_dict):
-    """Creates the .dem.rsc file string from key/value pairs of an OrderedDict
-
-    Output of function can be written to a file as follows
-        with open('my.dem.rsc', 'w') as f:
-            f.write(outstring)
-
-    Args:
-        rsc_dict (OrderedDict): data about dem in ordered key/value format
-            See `load_dem_rsc` output for example
-
-    Returns:
-        outstring (str) formatting string to be written to .dem.rsc
-
-    """
-    outstring = ""
-    rsc_dict = {k.lower(): v for k, v in rsc_dict.items()}
-    # for field, value in rsc_dict.items():
-    for field in RSC_KEYS:
-        # Make sure to skip extra keys that might be in the dict
-        if field not in RSC_KEYS:
-            continue
-
-        value = rsc_dict.get(field, DEFAULT_KEYS.get(field))
-        if value is None:
-            raise ValueError("%s is necessary for .rsc file: missing from dict" % field)
-
-        # Files seemed to be left justified with 14 spaces? Not sure why 14
-        # Apparently it was an old fortran format, where they use "read(15)"
-        if field in ('x_step', 'y_step'):
-            # give step floats proper sig figs to not output scientific notation
-            outstring += "{field:<14s}{val:0.12f}\n".format(field=field.upper(), val=value)
-        else:
-            outstring += "{field:<14s}{val}\n".format(field=field.upper(), val=value)
-
-    return outstring
-
-
-# TODO: see if this should be folded in, or just gdal loaded
-def load_elevation(filename):
-    """Loads a digital elevation map from either .hgt file or .dem
-
-    .hgt is the NASA SRTM files given. Documentation on format here:
-    https://dds.cr.usgs.gov/srtm/version2_1/Documentation/SRTM_Topo.pdf
-    Key point: Big-endian 2 byte (16-bit) integers
-
-    .dem is format used by Zebker geo-coded and ROI-PAC SAR software
-    Only difference is data is stored little-endian (like other SAR data)
-
-    Note on both formats: gaps in coverage are given by INT_MIN -32768,
-    so either manually set data(data == np.min(data)) = 0,
-        data = np.clip(data, 0, None), or when plotting, plt.imshow(data, vmin=0)
-    """
-    ext = os.path.splitext(filename)[1]
-    data_type = INT_16_LE if ext == '.dem' else INT_16_BE
-    data = np.fromfile(filename, dtype=data_type)
-    # Make sure we're working with little endian
-    if data_type == INT_16_BE:
-        data = data.astype(INT_16_LE)
-
-    # Reshape to correct size.
-    # Either get info from .dem.rsc
-    if ext == '.dem':
-        info = load_dem_rsc(filename)
-        dem_img = data.reshape((info['file_length'], info['width']))
-
-    # Or check if we are using STRM1 (3601x3601) or SRTM3 (1201x1201)
-    else:
-        if (data.shape[0] / 3601) == 3601:
-            # STRM1- 1 arc second data, 30 meter data
-            dem_img = data.reshape((3601, 3601))
-        elif (data.shape[0] / 1201) == 1201:
-            # STRM3- 3 arc second data, 90 meter data
-            dem_img = data.reshape((1201, 1201))
-        else:
-            raise ValueError("Invalid .hgt data size: must be square size 1201 or 3601")
-        # TODO: Verify that the min real value will be above -1000
-        min_valid = -1000
-        # Set NaN values to 0
-        dem_img[dem_img < min_valid] = 0
-
-    return dem_img
-
-
 def find_looks_taken(igram_path,
                      geo_path=None,
                      igram_dem_file="dem.rsc",
