@@ -35,7 +35,10 @@ def cli(ctx, verbose, path):
 
 # COMMAND: view-stack
 @cli.command('view-stack')
-@click.option("--filename", default='deformation.npy', help="Name of saved deformation stack")
+@click.option("--filename",
+              default='deformation.h5',
+              help="Name of saved file containing deformation stack")
+@click.option("--dset", default='stack/1', help="Dataset within hdf5 file")
 @click.option("--cmap", default='seismic', help="Colormap for image display.")
 @click.option("--label", default='Centimeters', help="Label on colorbar/yaxis for plot")
 @click.option("--title", help="Title for image plot")
@@ -48,23 +51,15 @@ def cli(ctx, verbose, path):
               is_flag=True,
               default=False)
 @click.pass_obj
-def view_stack(context, filename, cmap, label, title, row_start, row_end, col_start, col_end,
+def view_stack(context, filename, dset, cmap, label, title, row_start, row_end, col_start, col_end,
                rowcol):
-    """Explore timeseries on deformation image.
-
-    If deformation.npy and geolist.npy or .unw files are not in current directory,
-    use the --path option:
-
-        aper --path /path/to/igrams view_stack
-
+    """Explore timeseries on stack of deformation images.
     """
+    import apertools.sario, apertools.latlon, apertools.plotting
     import numpy as np
-    if filename.endswith(".h5"):
-        h_file = h5py.File(filename, "r")
-        deformation = h_file["deformation"]
-        geolist = [datetime.strptime(g.decode("ascii"), "%Y%m%d").date() for g in h_file["geolist"]]
-    else:
-        geolist, deformation = apertools.sario.load_deformation(context['path'], filename=filename)
+    with h5py.File(filename, "r") as f:
+        deformation = f[dset][:]
+        geolist = apertools.sario.load_geolist_from_h5(filename, dset)
 
     if geolist is None or deformation is None:
         return
@@ -74,7 +69,7 @@ def view_stack(context, filename, cmap, label, title, row_start, row_end, col_st
     else:
         rsc_data = apertools.sario.load(os.path.join(context['path'], 'dem.rsc'))
 
-    deformation = apertools.latlon.LatlonImage(data=deformation, dem_rsc=rsc_data)
+    deformation = apertools.latlon.LatlonImage(data=deformation, rsc_data=rsc_data)
     deformation = deformation[:, row_start:row_end, col_start:col_end]
     img = np.mean(deformation[-3:], axis=0)
 
@@ -262,7 +257,7 @@ def dem_rate(rsc_file, orig_rsc_file):
     if rsc_file is None:
         rsc_file = apertools.sario.find_rsc_file(directory=".")
 
-    x_uprate, y_uprate = apertools.utils.calc_upsample_rate(rsc_filename=rsc_file)
+    x_uprate, y_uprate = apertools.sario.calc_upsample_rate(rsc_filename=rsc_file)
 
     click.echo("%s has (%.5f, %.5f) times the default spacing in (x, y)" %
                (rsc_file, x_uprate, y_uprate))
@@ -271,9 +266,9 @@ def dem_rate(rsc_file, orig_rsc_file):
     click.echo("This is equal to (%.2f, %.2f) meter spacing between pixels" %
                (default_spacing / x_uprate, default_spacing / y_uprate))
     if orig_rsc_file is not None:
-        orig_x_uprate, orig_y_uprate = apertools.utils.calc_upsample_rate(
+        orig_x_uprate, orig_y_uprate = apertools.sario.calc_upsample_rate(
             rsc_filename=orig_rsc_file)
-        click.echo("(%d, %d) looks were taken on %s to get %s" %
+        click.echo("(%.0f, %.0f) looks were taken on %s to get %s" %
                    (orig_x_uprate / x_uprate, orig_y_uprate / y_uprate, orig_rsc_file, rsc_file))
 
 
@@ -356,8 +351,14 @@ def overlaps(sentinel_path, filename, path_num, start_date, end_date):
 @click.option("--rows", type=int, help="Number of rows (file_length) in file")
 @click.option("--dtype", help="Optional number dtype string")
 @click.option("--band", type=int, help="The band number to use for the VRT")
+@click.option(
+    "--interleave",
+    type=click.Choice(["BIP", "BIL", "BSQ"]),
+    default=None,
+    help="Type of pixel interleave in binary file",
+)
 @click.option("--num-bands", type=int, help="Number of bands in file")
-def save_vrt(filenames, rsc_file, cols, rows, dtype, band, num_bands):
+def save_vrt(filenames, rsc_file, cols, rows, dtype, band, interleave, num_bands):
     """Save GDAL .vrt file binary raster loading
 
     List as many filenames with the same rsc as necessary
@@ -371,15 +372,18 @@ def save_vrt(filenames, rsc_file, cols, rows, dtype, band, num_bands):
             dtype=dtype,
             rsc_file=rsc_file,
             band=band,
+            interleave=interleave,
             num_bands=num_bands,
         )
 
 
-# COMMAND: save-vrt
 @cli.command("smallslc")
 @click.argument("filenames", nargs=-1)
 @click.option("--rsc-file", help="If exists, the .rsc file of data")
-@click.option("--downrate", default=10, help="Downsampling beyond the 30m SRTM resolution")
+@click.option("--downrate",
+              default=10,
+              help="Downsampling beyond the 30m SRTM resolution",
+              show_default=True)
 def smallslc(
     filenames,
     rsc_file,
@@ -390,8 +394,7 @@ def smallslc(
     List as many filenames with the same rsc as necessary
     """
     import apertools.sario
-    import apertools.utils
-    x_uprate, y_uprate = apertools.utils.calc_upsample_rate(rsc_filename=rsc_file)
+    x_uprate, y_uprate = apertools.sario.calc_upsample_rate(rsc_filename=rsc_file)
     pct_x = int(100 / x_uprate / downrate)
     pct_y = int(100 / y_uprate / downrate)
     for f in filenames:
@@ -412,64 +415,47 @@ def smallslc(
         subprocess.check_call(cmd, shell=True)
 
 
+@cli.command("looked-dem")
+@click.option("--src-dem", default="../elevation.dem", help="Original, large DEM")
+@click.option("--dest-rsc", default="dem.rsc", help=".rsc file of the destination")
+@click.option("--outname", default="elevation_looked.dem")
+def looked_dem(src_dem, dest_rsc, outname):
+    """Save a smaller DEM version to match size of dest-rsc file
+
+    """
+    import apertools.sario as sario
+    rsc = sario.load(dest_rsc)
+    xstep, ystep = rsc["x_step"], rsc["y_step"]
+    # -r nearest == Use nearest-neighbor resampling, -tr = target resolution
+    cmd = f"gdal_translate -r nearest -of ROI_PAC -tr {xstep} {ystep} {src_dem} {outname}"
+    logger.info(cmd)
+    subprocess.check_call(cmd, shell=True)
+
+
 @cli.command("hdf5-gtiff")
 @click.argument("infile")
 @click.option("--output", "-o", help="output geotiff file")
 @click.option("--rsc", help=".rsc file containing lat/lon start and steps")
 @click.option("--dset", help="specify to only save one dataset")
-@click.option("--nodata", default="0")
-@click.option("--outtype", default="Float32")
+@click.option("--nodata", default="0", show_default=True)
+@click.option("--outtype", default="Float32", show_default=True)
 def geotiff(infile, rsc, output, dset, nodata, outtype):
+    from .hdf5_geotiff import hdf5_to_geotiff
+    return hdf5_to_geotiff(infile, rsc, output, dset, nodata, outtype)
+
+
+@cli.command("set-unit")
+@click.argument("filenames", nargs=-1)
+@click.option("--unit", "-u", default="cm", help="unit for file", show_default=True)
+def set_unit(filenames, unit):
     import apertools.sario
+    for f in filenames:
+        apertools.sario.set_unit(f, unit)
+
+
+@cli.command("az-inc-to-enu")
+@click.argument("infile")
+@click.option("--outfile", "-o", default="los_enu.tif", show_default=True)
+def convert_to_enu(infile, outfile):
     import apertools.utils
-    import apertools.kml
-
-    if rsc:
-        rsc_data = apertools.sario.load(rsc)
-    else:
-        try:
-            rsc_data = apertools.sario.load_dem_from_h5(infile)
-        except:
-            raise ValueError(".rsc loading failed")
-            # rsc_data = None
-
-    north, south, east, west = apertools.kml.rsc_nsew(rsc_data)
-    ullr_string = "%f %f %f %f" % (west, north, east, south)
-
-    if output is None:
-        file_ext = apertools.utils.get_file_ext(infile)
-        output = infile.replace(file_ext, ".tif")
-
-    # create_geotiff(rsc_data, kml_file, img_filename, shape='box', outfile)
-    # TODO: fix up the kml one...
-    cmd = 'gdal_translate -sds -a_nodata "{nodata}" -a_srs EPSG:4326 -a_ullr {ullr} {input} {out}'
-    if dset is None:
-        # tmp_out gets split per band
-        cmd1 = cmd.format(
-            ullr=ullr_string,
-            input=infile,
-            out="tmp_out.tif",
-            nodata=nodata,
-        )
-        logger.info("running:")
-        logger.info(cmd1)
-        subprocess.check_call(cmd1, shell=True)
-        cmd2 = 'gdal_merge.py -separate -ot {ot} -o {output} -n "{nd}" -a_nodata "{nd}" tmp_out*tif'.format(
-            nd=nodata,
-            ot=outtype,
-            output=output,
-        )
-        logger.info("running:")
-        logger.info(cmd2)
-        subprocess.check_call(cmd2, shell=True)
-        subprocess.check_call("rm tmp_out*.tif", shell=True)
-    else:
-        cmd1 = cmd.format(
-            ullr=ullr_string,
-            input=infile,
-            out=output,
-            nodata=nodata,
-        )
-        logger.info("running:")
-        logger.info(cmd1)
-        subprocess.check_call(cmd1, shell=True)
+    apertools.utils.az_inc_to_enu(infile, outfile)
