@@ -81,6 +81,8 @@ def take_looks(arr, row_looks, col_looks, separate_complex=False):
     """
     if row_looks == 1 and col_looks == 1:
         return arr
+    if isinstance(arr, dict):
+        return take_looks_rsc(arr, row_looks, col_looks)
     if np.iscomplexobj(arr) and separate_complex:
         mag_looked = take_looks(np.abs(arr), row_looks, col_looks)
         phase_looked = take_looks(np.angle(arr), row_looks, col_looks)
@@ -108,6 +110,24 @@ def _find_look_outsize(shape, row_looks, col_looks):
     return new_rows, new_cols
 
 
+def take_looks_rsc(rsc_data, row_looks, col_looks):
+    nrows, ncols = rsc_data["file_length"], rsc_data["width"]
+
+    out_rsc = rsc_data.copy()
+    out_rsc["x_step"] = rsc_data["x_step"] * col_looks
+    out_rsc["y_step"] = rsc_data["y_step"] * row_looks
+    out_rsc["file_length"] = nrows // row_looks
+    out_rsc["width"] = ncols // col_looks
+    return out_rsc
+
+
+def scale_dset(filename, dset, scale):
+    import h5py
+    with h5py.File(filename, "r+") as f:
+        data = f[dset]
+        data[...] *= scale
+
+
 def take_looks_gdal(outname, src_filename, row_looks, col_looks, format="ROI_PAC"):
     """Downsample an array on disk using gdal_translate
 
@@ -456,3 +476,24 @@ def get_cache_dir(force_posix=False, app_name="apertools"):
     if not os.path.exists(path):
         os.makedirs(path)
     return path
+
+
+def az_inc_to_enu(infile, outfile='geo_los_enu.tif'):
+    cmd = f'gdal_calc.py --quiet -A {infile} -B {infile} --A_band=1 --B_band=2 --outfile tmp_los_east.tif --calc="sin(deg2rad(A)) * cos(deg2rad(B+90))" '
+    print(cmd)
+    subprocess.run(cmd, check=True, shell=True)
+
+    cmd = f'gdal_calc.py --quiet -A {infile} -B {infile} --A_band=1 --B_band=2 --outfile tmp_los_north.tif --calc="sin(deg2rad(A)) * sin(deg2rad(B+90))" '
+    print(cmd)
+    subprocess.run(cmd, check=True, shell=True)
+
+    cmd = f'gdal_calc.py --quiet -A {infile} -B {infile} --A_band=1 --B_band=2 --outfile tmp_los_up.tif --calc="cos(deg2rad(A))" '
+    print(cmd)
+    subprocess.run(cmd, check=True, shell=True)
+
+    cmd = f'gdal_merge.py -separate -o {outfile} tmp_los_east.tif tmp_los_north.tif tmp_los_up.tif '
+    print(cmd)
+    subprocess.run(cmd, check=True, shell=True)
+    subprocess.run('rm -f tmp_los_east.tif tmp_los_north.tif tmp_los_up.tif',
+                   shell=True,
+                   check=True)
